#pragma once

#include <Astri_DAQ.h>
#include <Base_DAQ_Observer.h>
#include <hiredis/hiredis.h>

#include <atomic>
#include <thread>
#include <mutex>
#include <condition_variable>
#include <set>

namespace inaf::oasbo::DAQ_observers {
class AstriRedisDAQObserver: public BaseDAQ_Observer {
protected:
	inaf::oasbo::DAQ::AstriDAQ *astri_daq;
	redisContext *context = nullptr;
	std::string ip{};
	int port;

	bool checkConnThreadFlag = false;
	std::thread checkConn;
	bool isConnected();
	int connect();

    std::set<std::string> keys;
	bool stopFlag = false;
	std::mutex writeMutex;
	std::thread observerThread;
	std::condition_variable sleep_cv;
	int updateInterval;
	void updateThread();
	void updateDAQStats();

	static std::string lastopenfile_key;
	static std::string lastclosedfile_key;
	static std::string mode_key;
	static std::string receiverSource_key;
	static std::string receiverConn_key;
	static std::string provider_destination_key;

public:
	AstriRedisDAQObserver(inaf::oasbo::DAQ::BaseDAQ &dataAcquisition,std::string ip, int port);
	~AstriRedisDAQObserver();

	void updatePacketStats() override;
	void updateArchiverStats() override;
	void updateProviderStats() override;
	void updateReceiverStats() override;
	void updateAll() override;
	void start() override;
	void stop() override;
};
}
